from importlib.metadata import version

import rich_click as click
from rich.console import Console
from rich.table import Table

from pycnpj_cpf.core import cnpj_or_cpf_is_valid

click.rich_click.USE_RICH_MARKUP = True
click.rich_click.USE_MARKDOWN = True
click.rich_click.SHOW_ARGUMENTS = True
click.rich_click.GROUP_ARGUMENTS_OPTIONS = True
click.rich_click.SHOW_METAVARS_COLUMN = True
click.rich_click.APPEND_METAVARS_HELP = True


@click.group()
@click.version_option(version('pycnpj-cpf'))
def main():
    """CNPJ and CPF validator.
    This cli application validates if the CNPJ/CPF entered is valid.
    """


@main.command()
@click.option('-v', '--value', help='Value to be validated.', required=True)
def validate(value: str) -> bool:
    table = Table(title='CNPJ and CPF validator')
    headers = ['CNPJ/CPF', 'Is Valid?']
    for header in headers:
        table.add_column(header, style='magenta')

    result = cnpj_or_cpf_is_valid(value)
    table.add_row(f'{value}', str(result))
    console = Console()
    console.print(table)



import pytest

from pycnpj_cpf.core import remove_punctuation


@pytest.mark.unit
@pytest.mark.high
def test_positive_remove_punctuation_cnpj():
    """Positive test for function remove_punctuation on a cnpj."""
    assert remove_punctuation('50.822.716/0001-42') == '50822716000142'


@pytest.mark.unit
@pytest.mark.high
def test_positive_remove_punctuation_cpf():
    """Positive test for function remove_punctuation on a cpf."""
    assert remove_punctuation('256.152.150-89') == '25615215089'



import pytest

MARKER = """\
unit: Mark unit tests
integration: Mark integration tests
high: High Priority
medium: Medium Priority
low: Low Priority
"""


def pytest_configure(config):
    for line in MARKER.split("\n"):
        config.addinivalue_line("markers", line)


@pytest.fixture(autouse=True)
def go_to_tmpdir(request):
    tmpdir = request.getfixturevalue("tmpdir")
    with tmpdir.as_cwd():
        yield



from re import match, sub

TWO = 2
NUMBER_FOR_REST = 11
SIZE_CPF = 11
SIZE_CNPJ = 14
PATTERN = r'[.\-/]'

CPF = {
    'first_digit': [10, 9, 8, 7, 6, 5, 4, 3, 2],
    'second_digit': [11, 10, 9, 8, 7, 6, 5, 4, 3, 2],
}

CNPJ = {
    'first_digit': [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2],
    'second_digit': [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2],
}

VALUE_PATTERN = (
    r'^[0-9]{11}$|^[0-9]{3}.[0-9]{3}.[0-9]{3}-[0-9]{2}$'
    r'|^[0-9A-Z]{12}[0-9]{2}$'
    '|^[0-9A-Z]{2}.[0-9A-Z]{3}.[0-9A-Z]{3}/[0-9A-Z]{4}-[0-9]{2}$'
)


def verify_pattern(value: str) -> bool:
    if (
        bool(match(VALUE_PATTERN, value))
        and len(set(sub(PATTERN, '', value))) > 1
    ):
        return True
    else:
        raise ValueError


def remove_punctuation(value: str) -> str:
    return sub(PATTERN, '', value)


def first_digit_cpf_checker_is_valid(value: str) -> bool:
    value = remove_punctuation(value)
    combinations = list(
        zip(CPF.get('first_digit'), [int(x) for x in value[:10]])
    )
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = (
        0 if rest_of_division < TWO else NUMBER_FOR_REST - rest_of_division
    )
    return result == int(value[9])


def second_digit_cpf_checker_is_valid(value: str) -> bool:
    value = remove_punctuation(value)
    combinations = list(
        zip(CPF.get('second_digit'), [int(x) for x in value[:11]])
    )
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = (
        0 if rest_of_division < TWO else NUMBER_FOR_REST - rest_of_division
    )
    return result == int(value[10])


def first_digit_cnpj_checker_is_valid(value: str) -> bool:
    value = remove_punctuation(value)
    combinations = list(
        zip(CNPJ.get('first_digit'), [ord(x) - 48 for x in value[:12]])
    )
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = (
        0 if rest_of_division < TWO else NUMBER_FOR_REST - rest_of_division
    )
    return result == int(value[12])


def second_digit_cnpj_checker_is_valid(value: str) -> bool:
    value = remove_punctuation(value)
    combinations = list(
        zip(CNPJ.get('second_digit'), [ord(x) - 48 for x in value[:13]])
    )
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = (
        0 if rest_of_division < TWO else NUMBER_FOR_REST - rest_of_division
    )
    return result == int(value[13])


def cnpj_or_cpf_is_valid(value: str) -> bool:
    result = False

    try:
        verify_pattern(value)

        value = remove_punctuation(value)

        if len(value) == SIZE_CPF and (
            first_digit_cpf_checker_is_valid(value)
            and second_digit_cpf_checker_is_valid(value)
        ):
            result = True
        if len(value) == SIZE_CNPJ and (
            first_digit_cnpj_checker_is_valid(value)
            and second_digit_cnpj_checker_is_valid(value)
        ):
            result = True
    except ValueError:
        result = False
    return result



import pytest
from click.testing import CliRunner

from pycnpj_cpf.cli import main, validate

cmd = CliRunner()


@pytest.mark.integration
@pytest.mark.high
@pytest.mark.parametrize(
    "value",
    [
        "848.044.710-90",
        "998.817.900-69",
        "895.761.04050",
        "71.418.067/0001-99",
        "96971115000100",
        "99.696.7540001-95",
    ],
)
def test_positive_call_validate_command(value):
    """Test command validate."""
    result = cmd.invoke(validate, ["--value", value])
    assert result.exit_code == 0


@pytest.mark.integration
@pytest.mark.medium
@pytest.mark.parametrize(
    "wrong_command", ["val", "validaty", "verify", "check"]
)
def test_negative_call_validate_command(wrong_command):
    """Test command validate."""
    result = cmd.invoke(main, wrong_command, "--value=96971115000100")
    assert result.exit_code != 0
    assert f"No such command '{wrong_command}'." in result.output
